#29 Oct 2016
#copula model in the case of gamma mixed model (longitudinal data) for creatinine data (Fieuws et al., 2008, Biostatistics):

rm(list=ls(all=TRUE))

runif(1)
seed<-.Random.seed[1]
set.seed(seed, kind ="default")

library(copula)
library(MASS)

data.y<-read.table(header=TRUE,"y.creat.txt")  #read the outcome at 4 time points
attach(data.y)
summary(data.y)

data.cov<-read.table(header=TRUE,"NewDataCov.txt") # read the covariates at 4 time points
attach(data.cov)
summary(data.cov)

loglik.m <- function(theta1, y, x) {
  l <- length(theta1)
  n <- length(y)
  bi<-rnorm(n,mean=0,sd=sqrt(theta1[l]))
  sum(dgamma(y, shape= theta1[l-1], rate=(1/theta1[l-1])*exp(x %*% theta1[1:4]+bi), log = FALSE)) }

loglik.cop <- function(u, myCop) {
  sum(log(dCopula(u, myCop))) 
}

probtrans.m <- function(theta1, y, x) {
  l <- length(theta1)
  n <- length(y)
  bi<-rnorm(n,mean=0,sd=sqrt(theta1[l]))
  pgamma(y, shape= theta1[l-1], rate=(1/theta1[l-1])*exp(x %*% theta1[1:4]+bi), log.p = FALSE) 
}

myloglik <- function(theta, dataRan, myCop) {
  dataRan=data.frame(y,xmat)
  
  l1 <- ncol(xmat[[1]]) + 2
  theta1 <- theta[1:l1]
  
  a <- theta[-(1:l1)]
  u <- cbind(probtrans.m(theta1, y[, 1], xmat[[1]]), probtrans.m(theta1, y[, 2], xmat[[2]]),  
             probtrans.m(theta1, y[, 3], xmat[[3]]), probtrans.m(theta1, y[, 4], xmat[[4]]) )
  u<-ifelse(u>=0.99,0.95,u)
  u<-ifelse(u==0,0.001,u)
  
  myCop@parameters <- a
  loglik <- loglik.m(theta1, y[, 1], xmat[[1]]) + loglik.m(theta1, y[, 2], xmat[[2]]) + 
    loglik.m(theta1, y[, 3], xmat[[3]]) + loglik.m(theta1, y[, 4], xmat[[4]]) +
    loglik.cop(u, myCop)
  loglik
}
###########################
n=1111
m=4

y<-data.y[,2:5]
X1<-cbind(male, BMI.t1, BPsys.t1)
X2<-cbind(male, BMI.t2, BPsys.t2)
X3<-cbind(male, BMI.t3, BPsys.t3)
X4<-cbind(male, BMI.t4, BPsys.t4)

xmat <- list( model.matrix(~X1),model.matrix(~X2), model.matrix(~X3),model.matrix(~X4) )
  
dataCoup=data.frame(y=y, xmat=xmat)

###########
theta1<-c(0.48, -0.06, 0.003, -0.0002, 1.6, 0.02 ) # (intercept, male, BMI, BPsys, shape, sigma2b) #initial parameters
a <-0.0           #copula parameter
theta<-c(theta1,a)
myCop <- normalCopula(a, dim = m, dispstr ="ar1")

myloglik(theta=theta, dataRan=dataCoup, myCop=myCop)
######To use optim function, we only depend the fucntion to the theta###############
myloglik1 <- function(theta) {
  l1 <- ncol(xmat[[1]]) + 2
  theta1 <- theta[1:l1]
  
  u <- cbind(probtrans.m(theta1, y[, 1], xmat[[1]]), probtrans.m(theta1, y[, 2], xmat[[2]]),  
             probtrans.m(theta1, y[, 3], xmat[[3]]), probtrans.m(theta1, y[, 4], xmat[[4]]))

    u<-ifelse(u>=0.99,0.99,u)
   u<-ifelse(u==0,0.001,u)
    
  a1 <- theta[-(1:l1)]
  myCop1 <- normalCopula(a1, dim = m, dispstr = "ar1")
  
  loglik <- loglik.m(theta1, y[, 1], xmat[[1]]) + loglik.m(theta1, y[, 2], xmat[[2]]) + 
            loglik.m(theta1, y[, 3], xmat[[3]]) + loglik.m(theta1, y[, 4], xmat[[4]]) +
            loglik.cop(u, myCop1)
  loglik
}
####################################################################################
fit <- optim(theta, myloglik1, method="L-BFGS-B", lower=c(.45,-.08,.001,-.0003,  1.3,0.01,0.01), 
             upper=c(.50,-.02,.003,-.0001, 2, .05, 0.5), hessian = TRUE, control=list(fnscale=-1))

Est<-fit$par
Est

Var<-solve(fit$hessian)
sqrt(diag(Var))
Var
#--------------------------------------------------