#20 Nov 2017

rm(list=ls(all=TRUE))

runif(1)
seed<-.Random.seed[1]
set.seed(seed, kind ="default")

library(nlme)
library(mvtnorm)
library(msm)
#---------------
# square-root-inverse of a matrix
mat.sqrt<-function(A)
{
  ei<-eigen(A)
  d<-ei$values
  d<-(d+abs(d))/2
  d2<-sqrt(d)
  d2[d == 0]<-0
  ans<-ei$vectors %*% diag(d2) %*% t(ei$vectors)
  return(ans)
}
####################################################
n <- 200 #number of subjects
m=5  #number of follow ups
#True parameters:
beta <- c(1, 0.5)  #regression coefficents and variance from marginal model (b0,b1)
sigma2b=1
sigma2e=0.5  #0.5, 1
ro=0.6 #0,0.3,0.6             #correlation parameter

#covariate tij
tij<-matrix(0,n,m)
for(i in 1:n){
  for(j in 1:m){
    tij[i,j]<-(j-1)/10
  }
}

a1<-1
b1<-m
id<-rep(0,m*n)
timej<-rep(0,m*n)
for(i in 1:n){
  id[a1:b1]<-i
  timej[a1:b1]<-c(0,0.1,0.2,0.3,0.4)
  a1<-b1+1
  b1<-a1+m-1
}

Sigma.e<-matrix(0,m,m)
for(i in 1:m){
  for(j in 1:m){
    Sigma.e[i,j]<-(sigma2e)*ro^{abs(i-j)}
  }
}
mu.e<-rep(0,m)
I.e<-diag(1,m)
Sigmae.sqrt<-mat.sqrt(Sigma.e) #%*%Sigma.e

B=5000

BETA.pro<-matrix(0,B,2)
VAR.BETA.pro<-matrix(0,B,2)
SIGMA2E.pro<-rep(0,B)
SIGMA2B.pro<-rep(0,B)
RO.pro<-rep(0,B)

BETA.nav<-matrix(0,B,2)
VAR.BETA.nav<-matrix(0,B,2)
SIGMA2E.nav<-rep(0,B)
SIGMA2B.nav<-rep(0,B)

SE.BETA.pro<-matrix(0,B,2)
SE.SIGMA2B.pro<-rep(0,B)
SE.SIGMA2E.pro<-rep(0,B)
SE.RO.pro<-rep(0,B)

SE.BETA.nav<-matrix(0,B,2)
SE.SIGMA2B.nav<-rep(0,B)
SE.SIGMA2E.nav<-rep(0,B)

for(b in 1:B){
y.mat<-matrix(0,n,m)
rnom1<-rnorm(n,0,1)
for(i in 1:n){
e.b<-rmvnorm(1,mu.e,I.e)
temi<-Sigmae.sqrt%*%t(e.b)
for(j in 1:m){
y.mat[i,j]<-beta[1]+beta[2]*(j-1)/10+sqrt(sigma2b)*rnom1[i]+temi[j]
}
}

y.ran<-rep(0,m*n)
a1<-1
b1<-m
for(i in 1:n){
  y.ran[a1:b1]<-y.mat[i,]
  a1<-b1+1
  b1<-a1+m-1
}

dataRan=data.frame(y.ran=y.ran, id=id, timej=timej)

#proposed approah:
fit.pro<-lme(y.ran ~ timej, 
    random = ~ 1 | id, 
    correlation = corAR1(ro),
    data=dataRan, control = lmeControl(msMaxIter = 500, maxIter=500), method="ML") #correct (proposed) model

inte.pro<-intervals(fit.pro)

BETA.pro[b,]<-fit.pro$coefficients$fixed
SIGMA2E.pro[b]<-(fit.pro$sigma)^2
SIGMA2B.pro[b]<-as.numeric(VarCorr(fit.pro)[,"Variance"][1])
RO.pro[b]<-coef(fit.pro$modelStruct$corStruct,unconstrained=FALSE)

SE.BETA.pro[b,]<-c( (inte.pro$fixed[1,3]-inte.pro$fixed[1,2])/1.96,(inte.pro$fixed[2,3]-inte.pro$fixed[2,2])/1.96 ) 
#1 SE.SIGMA2B.pro[b]<-as.numeric((inte.pro$reStruct$id[3]-inte.pro$reStruct$id[2])/1.96)
#3 SE.SIGMA2E.pro[b]<-(inte.pro$sigma[3]-inte.pro$sigma[2])/1.96
SE.RO.pro[b]<-(inte.pro$corStruct[3]-inte.pro$corStruct[2])/1.96

var<-fit.pro$apVar
#var
par<-attr(var,"Pars")
#par
vc<-exp(par)^2
#vc
SE.SIGMA2B.pro[b]<-as.numeric(deltamethod(~exp(x1)^2,par,var))
SE.SIGMA2E.pro[b]<-as.numeric(deltamethod(~exp(x3)^2,par,var))


#navie approach:
fit.nav<-lme( y.ran ~ timej, data =dataRan, random = ~ 1 | id , control = lmeControl(msMaxIter = 500, maxIter=500), method="ML") #naive model
inte.nav<-intervals(fit.nav)

BETA.nav[b,]<-fit.nav$coefficients$fixed
SIGMA2E.nav[b]<-(fit.nav$sigma)^2
SIGMA2B.nav[b]<-as.numeric(VarCorr(fit.nav)[,"Variance"][1])

SE.BETA.nav[b,]<-c( (inte.nav$fixed[1,3]-inte.nav$fixed[1,2])/1.96,(inte.nav$fixed[2,3]-inte.nav$fixed[2,2])/1.96 ) 
#1 SE.SIGMA2B.nav[b]<-as.numeric((inte.nav$reStruct$id[3]-inte.nav$reStruct$id[2])/1.96)
#3 SE.SIGMA2E.nav[b]<-(inte.nav$sigma[3]-inte.nav$sigma[2])/1.96

var.nav<-fit.nav$apVar
#var.nav
par.nav<-attr(var.nav,"Pars")
#par.nav
vc.nav<-exp(par.nav)^2

SE.SIGMA2B.nav[b]<-as.numeric(deltamethod(~exp(x1)^2,par.nav,var.nav))
SE.SIGMA2E.nav[b]<-as.numeric(deltamethod(~exp(x2)^2,par.nav,var.nav))

cat(b)
}#loop of B

apply(BETA.pro,2,mean)
summary(SIGMA2B.pro)
summary(SIGMA2E.pro)
summary(RO.pro)

apply(SE.BETA.pro,2,mean)
summary(SE.SIGMA2B.pro)
summary(SE.SIGMA2E.pro)
summary(SE.RO.pro)

#simulated SE:
apply(BETA.pro,2,sd)
sd(SIGMA2B.pro)
sd(SIGMA2E.pro)
sd(RO.pro)

##########naive:
apply(BETA.nav,2,mean)
summary(SIGMA2B.nav)
summary(SIGMA2E.nav)

apply(SE.BETA.nav,2,mean)
summary(SE.SIGMA2B.nav)
summary(SE.SIGMA2E.nav)

#simulated SE:
apply(BETA.nav,2,sd)
sd(SIGMA2B.nav)
sd(SIGMA2E.nav)
############################
