#27 Nov 2017

rm(list=ls(all=TRUE))
################################### Generating Longitudinal outcomes ##############################################
library(foreign)
library(copula)
library(mvtnorm)
library(MASS)

set.seed(6)

cat("the program started:",date(),"\n \n")
############## set-up ###########################
n=200 #number of subjects
R=1000  #number of simulaitons
nr<-n*R
T<-5 #number of follow-ups
t<-c(1:T)

beta_0<-1 
beta_1<-2 
zeta=1
sigma2=1 
rho<-0.9 #copula parameter
####### generating dummy matrices for mean ########
x1<-matrix(rep(NaN,n),nrow=n,ncol=T)

for ( i in 1:n){
  for (j in 1:T)
  {
    x1[i,j]<-(j)/10 #runif(1,0,2)
  }
  }
####################################################################################################################
loglik.m <- function(theta1, y.ran, x) {
  l <- length(theta1)
  n <- length(y.ran)
  bi<-rnorm(n,mean=0,sd=sqrt(theta1[l]))
  sum(dlogis(y.ran, location=x %*% theta1[1:2]+bi, scale=theta1[l-1], log = FALSE)) 
                                       }

loglik.cop <- function(u, myCop) {
  #myCop@parameters <- a
  sum(log(dCopula(u, myCop))) 
                                    }

probtrans.m <- function(theta1, y.ran, x) {
  l <- length(theta1)
  n <- length(y.ran)
  bi<-rnorm(n,mean=0,sd=sqrt(theta1[l]))
  plogis(y.ran, location=x %*% theta1[1:2]+bi, scale= theta1[l-1], log.p = FALSE) 
}

myloglik <- function(theta, dataRan, myCop) {
  dataRan=data.frame(y.ran,xmat)
  
  l1 <- ncol(xmat[[1]]) + 2
  theta1 <- theta[1:l1]
  
  a <- theta[-(1:l1)]
  u <- cbind(probtrans.m(theta1, y.ran[, 1], xmat[[1]]), probtrans.m(theta1, y.ran[, 2], xmat[[2]]),  
             probtrans.m(theta1, y.ran[, 3], xmat[[3]]), probtrans.m(theta1, y.ran[, 4], xmat[[4]]),
            probtrans.m(theta1, y.ran[, 5], xmat[[5]]) )
  
  #u<-ifelse(u>=0.99,0.5,u)
  
  myCop@parameters <- a
  
  loglik <- loglik.m(theta1, y.ran[, 1], xmat[[1]]) + loglik.m(theta1, y.ran[, 2], xmat[[2]]) + 
    loglik.m(theta1, y.ran[, 3], xmat[[3]])+ loglik.m(theta1, y.ran[, 4], xmat[[4]]) 
    +loglik.m(theta1, y.ran[, 5], xmat[[5]]) 
    + loglik.cop( u, myCop)
  loglik
}

genY <- function(theta, xmat, myCop) {
  l <- ncol(xmat[[1]]) + 2
  theta1 <- theta[1:l]
  
  a <- theta[-(1:l)]
  n <- nrow(xmat[[1]])
  u <- rCopula(n,myCop)
  bi<-rnorm(n,mean=0,sd=sqrt(theta1[l]))
  y1 <- qlogis(u[, 1], location=xmat[[1]] %*% theta1[1:2]+bi, scale=theta1[l-1], log.p=FALSE )
  y2 <- qlogis(u[, 2], location=xmat[[2]] %*% theta1[1:2]+bi, scale=theta1[l-1], log.p=FALSE )
  y3 <- qlogis(u[, 3], location=xmat[[3]] %*% theta1[1:2]+bi, scale=theta1[l-1], log.p=FALSE )
  y4 <- qlogis(u[, 4], location=xmat[[4]] %*% theta1[1:2]+bi, scale=theta1[l-1], log.p=FALSE )
  y5 <- qlogis(u[, 5], location=xmat[[5]] %*% theta1[1:2]+bi, scale=theta1[l-1], log.p=FALSE ) 
  cbind(y1, y2, y3, y4, y5) }
####################################################

####################################################
p=5  #number of model parameters
#True parameters:
theta1 <- c(beta_0, beta_1, zeta, sigma2)  #regression coefficents and variance from marginal model (b0,b1,zeta,sigma2b)
theta <- c(theta1,rho)

myCop <- ellipCopula(family = "normal", dim = T, param = rho, dispstr = "ar1")
xmat <- list( model.matrix(~x1[,1]),model.matrix(~x1[,2]), model.matrix(~x1[,3]),model.matrix(~x1[,4]),
              model.matrix(~x1[,5]) )
#####################
Est.sim<-matrix(0,R,p)
Var.sim<-matrix(0,R,p*p)
NUM<-rep(0,R)
#####################
p1=4
Est.sim.nav<-matrix(0,R,p1)
Var.sim.nav<-matrix(0,R,p1*p1)
NUM.nav<-rep(0,R)
######################

for(r in 1:R){
  
  y.ran <- genY(theta, xmat, myCop)
  dataRan=data.frame(y.ran=y.ran, xmat=xmat)
  myloglik(theta=theta, dataRan=dataRan, myCop=myCop)
  ######To use optim function, we only depend the fucntion to the theta###############
  myloglik1 <- function(theta) {
    l1 <- ncol(xmat[[1]]) + 2
    theta1 <- theta[1:l1]
    
    u <- cbind(probtrans.m(theta1, y.ran[, 1], xmat[[1]]), probtrans.m(theta1, y.ran[, 2], xmat[[2]]),  
               probtrans.m(theta1, y.ran[, 3], xmat[[3]]), probtrans.m(theta1, y.ran[, 4], xmat[[4]]),
               probtrans.m(theta1, y.ran[, 5], xmat[[5]]) )
    #u<-ifelse(u>=0.99,0.5,u)
    
    a1 <- theta[-(1:l1)]
    # myCop@parameters <- a
    myCop1 <- ellipCopula(family = "normal", dim = T, param = a1, dispstr = "ar1") 
    
     
    loglik <- loglik.m(theta1, y.ran[, 1], xmat[[1]]) + loglik.m(theta1, y.ran[, 2], xmat[[2]]) + 
      loglik.m(theta1, y.ran[, 3], xmat[[3]]) + loglik.m(theta1, y.ran[, 4], xmat[[4]]) 
      +loglik.m(theta1, y.ran[, 5], xmat[[5]]) 
      + loglik.cop(u, myCop1)
    loglik
  }
  ####################################################################################
  tryCatch(
    {
      # print(r)
      
      fit <- optim(theta, myloglik1, method="L-BFGS-B", lower=c(0.001,0.001,0.01,0.01,0.01), 
                   upper=c(2,3,2,2,1), hessian = TRUE, control=list(fnscale=-1, trace=1)) #, factr=0.1)) #control = list(maxit=1000),trace=1
      
      # stop("Urgh, the iphone is in the blender !")
    },
    error=function(e){cat("ERROR :",conditionMessage(e), "\n")})

  if(fit$counts[1]>1000 | fit$counts[2]>1000){NUM[r]<-1}
  Est.sim[r,]<-fit$par
  Var.tem<-ginv(fit$hessian)
  
  Var.sim[r,1:p]<-Var.tem[1,]
  Var.sim[r,(p+1):(2*p)]<-Var.tem[2,]
  Var.sim[r,(2*p+1):(3*p)]<-Var.tem[3,]
  Var.sim[r,(3*p+1):(4*p)]<-Var.tem[4,]
  Var.sim[r,(4*p+1):(5*p)]<-Var.tem[5,]
 #-------------------------------------
  ######To use optim function, we only depend the fucntion to the theta###############
  myloglik1.nav <- function(theta) {
    l1 <- ncol(xmat[[1]]) + 2
    theta1 <- theta[1:l1]
    
    # u <- cbind(probtrans.m(theta1, y.ran[, 1], xmat[[1]]), probtrans.m(theta1, y.ran[, 2], xmat[[2]]),  
    #             probtrans.m(theta1, y.ran[, T], xmat[[T]])) #, probtrans.m(theta1, y[, 4], xmat[[4]]),
    # probtrans.m(theta1, y[, 5], xmat[[5]]) )
    #u<-ifelse(u>=0.99,0.5,u)
    
    #a1 <- theta[-(1:l1)]
    # myCop@parameters <- a
    # myCop1 <- ellipCopula(family = "normal", dim = T, param = a1) 
    
    
    loglik <- loglik.m(theta1, y.ran[, 1], xmat[[1]]) + loglik.m(theta1, y.ran[, 2], xmat[[2]]) + 
      loglik.m(theta1, y.ran[, 3], xmat[[3]])+ loglik.m(theta1, y.ran[, 4], xmat[[4]]) 
    +loglik.m(theta1, y.ran[, 5], xmat[[5]]) 
    #+ loglik.cop(u, myCop1)
    loglik
  }
  ####################################################################################
  tryCatch(
    {
      # print(r)
      
      fit.nav <- optim(theta1, myloglik1.nav, method="L-BFGS-B", lower=c(0.001,0.001,0.01,0.01), 
                   upper=c(2,4,2,2), hessian = TRUE, control=list(fnscale=-1, trace=1)) #, factr=0.1)) #control = list(maxit=1000),trace=1
      
      # stop("Urgh, the iphone is in the blender !")
    },
    error=function(e){cat("ERROR :",conditionMessage(e), "\n")})
  
  if(fit.nav$counts[1]>1000 | fit.nav$counts[2]>1000){NUM.nav[r]<-1}
  Est.sim.nav[r,]<-fit.nav$par
  Var.tem.nav<-ginv(fit.nav$hessian)
  
  Var.sim.nav[r,1:p1]<-Var.tem.nav[1,]
  Var.sim.nav[r,(p1+1):(2*p1)]<-Var.tem.nav[2,]
  Var.sim.nav[r,(2*p1+1):(3*p1)]<-Var.tem.nav[3,]
  Var.sim.nav[r,(3*p1+1):(4*p1)]<-Var.tem.nav[4,]
   cat(r)
}#end of loop.
apply(Est.sim,2,mean)
#=============================================================================
Var.tem<-apply(Var.sim,2,mean)  #Model-based SE
Var.est<-rep(0,p)
Var.est[1]<-Var.tem[1]
Var.est[2]<-Var.tem[7]
Var.est[3]<-Var.tem[13]
Var.est[4]<-Var.tem[19]
Var.est[5]<-Var.tem[25]
se.est<-sqrt(Var.est)
Var.est
se.est

#Emprirical SD
sd<-c(sd(Est.sim[,1]), sd(Est.sim[,2]), sd(Est.sim[,3]), sd(Est.sim[,4]), sd(Est.sim[,5]))
sd
##############################################################
##########################NAIVE APPROACH####################################
####################################################
apply(Est.sim.nav,2,mean)
#=============================================================================
Var.tem.nav<-apply(Var.sim.nav,2,mean)  #Model-based SE
Var.est.nav<-rep(0,p1)
Var.est.nav[1]<-Var.tem.nav[1]
Var.est.nav[2]<-Var.tem.nav[6]
Var.est.nav[3]<-Var.tem.nav[11]
Var.est.nav[4]<-Var.tem.nav[16]
se.est.nav<-sqrt(Var.est.nav)
Var.est.nav
se.est.nav

#Emprirical SD
sd.nav<-c(sd(Est.sim.nav[,1]), sd(Est.sim.nav[,2]), sd(Est.sim.nav[,3]), sd(Est.sim.nav[,4]))
sd.nav
##############################################################
