#3 Nov 2016
#Naive model in the case of gamma mixed model (longitudinal data)
#Simulation study:

rm(list=ls(all=TRUE))

library(copula)
library(MASS)
library(lme4)

set.seed(6)
 ####################################################
 #Now, we generate a sample of size n = 200. The design matrices are generated from continuous normal variables.
 n <- 200 #number of subjects
 m=5  #number of follow ups
 p=4  #number of model parameters
 #True parameters:
 theta <- c(1, -1, 2, 0.02)  #regression coefficents and variance from marginal model (b0,b1,alpha,sigma2b)
 
 a=0.20
 myCop <- normalCopula(a, dim = m, dispstr ="ar1")
 
 xij<-matrix(0,n,m)
 for(i in 1:n){
   xij[i,1:m]<-runif(m,0,2)
              }

 x.long<-rep(0,n*m)
 subject.long<-rep(0,n*m)
 a<-1
 b<-m
 for(i in 1:n){
   x.long[a:b]<-as.numeric(xij[i,])
   subject.long[a:b]<-rep(i,m)
   a<-b+1
   b<-a+m-1
    }
  
 #To illustrate, we define a function to generate response variables from given parameter vector, design matrices, and copula structure:
 genY <- function(theta, xij, myCop) {
   n <- 200 
   u <- rCopula(n,myCop)
   bi<-rnorm(n,mean=0,sd=sqrt(theta[4]))
   y1 <- qgamma(u[, 1], shape =theta[3], rate=(1/theta[3])*exp(theta[1]+theta[2]*xij[,1]+bi), log.p=FALSE )
   y2 <- qgamma(u[, 2], shape =theta[3], rate=(1/theta[3])*exp(theta[1]+theta[2]*xij[,2]+bi), log.p=FALSE )
   y3 <- qgamma(u[, 3], shape =theta[3], rate=(1/theta[3])*exp(theta[1]+theta[2]*xij[,3]+bi), log.p=FALSE )
   y4 <- qgamma(u[, 4], shape =theta[3], rate=(1/theta[3])*exp(theta[1]+theta[2]*xij[,4]+bi), log.p=FALSE )
   y5 <- qgamma(u[, 5], shape =theta[3], rate=(1/theta[3])*exp(theta[1]+theta[2]*xij[,5]+bi), log.p=FALSE )
   cbind(y1, y2, y3, y4,y5) }
  
 R=1000  #number of simulaitons
 Est.sim<-matrix(0,R,p)
 SE.sim<-matrix(0,R,p)
 NUM<-rep(0,R)
 
 for(r in 1:R){
   y.mat <- genY(theta, xij, myCop)
   
 y.long<-rep(0,n*m)
 a<-1
 b<-m
 for(i in 1:n){
   y.long[a:b]<-as.numeric(y.mat[i,])
   a<-b+1
   b<-a+m-1
              }
   
 data.long<-data.frame(subject.long,x.long,y.long) 
 
 res<-glmer(y.long ~ 1 +  x.long + (1|subject.long), data=data.long, family=Gamma(link = log))
 
 ttt<-print(VarCorr(res),comp=c("Variance","Std.Dev."))
 fit.tt<-as.data.frame(ttt)
 alpha.hat<-1/fit.tt$vcov[2]  
 sigma2b.hat<-fit.tt$vcov[1]
 sd.sigma2b<-fit.tt$sdcor[1]  
 sd.alpha<-(fit.tt$sdcor[2])*((fit.tt$vcov[2])^2) 
   
 res.table <- as.data.frame(coef(summary(res)))
 
 Est.sim[r,]<-c(res.table$Estimate, alpha.hat, sigma2b.hat)
 SE.sim[r,]<-c(res.table$`Std. Error`, sd.alpha, sd.sigma2b)
 cat(r)
 }#end of loop.
 apply(Est.sim,2,mean)  #average of model parameters estimate
 apply(SE.sim,2,mean) #model-based SE
 sqrt(apply(Est.sim,2,var)) #empirical SE
 #=============================================================================
 